<?php

namespace Drupal\group_subscription\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

if (!defined('GROUP_SUBSCRIPTION_TYPE_DISABLED')) {
  define('GROUP_SUBSCRIPTION_TYPE_DISABLED', 0);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE')) {
  define('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE', 1);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_DAILY')) {
  define('GROUP_SUBSCRIPTION_TYPE_DAILY', 2);
}

/**
 * Defines a block form to capture user settings for a specific group.
 */
class GroupSubscriptionBlockForm extends FormBase {

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The group entity.
   *
   * @var \Drupal\group\Entity\GroupInterface
   */
  protected $group;

  /**
   * Constructs a new GroupSubscriptionBlockForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $current_path = \Drupal::service('path.current')->getPath();
    $path_parts = explode('/', $current_path);

    if ($path_parts[1] === 'group') {
      $this->group = $this->entityTypeManager->getStorage('group')->load($path_parts[2]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'group_subscription_block_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // We don't want to build anything unless we are viewing a group entity.
    if ($this->group) {
      $user = \Drupal::currentUser();

      // See if the current user is a member of the group we are viewing.
      if ($this->group->getMember($user)) {
        // Retrieve group subscription configuration for this user / group.
        $result = \Drupal::database()->select('group_subscription_groups', 'g');
        $result->fields('g', ['uid', 'gid', 'groupname', 'subscription']);
        $result->condition('g.uid', $user->id());
        $result->condition('g.gid', $this->group->id());
        $group_settings = $result->execute()->fetchObject();

        if ($group_settings) {
          $group_settings = (array) $group_settings;
          $subscription = $group_settings['subscription'];
        }
        else {
          $subscription = GROUP_SUBSCRIPTION_TYPE_DISABLED;
        }

        // Store the current user id.
        $form['uid'] = [
          '#type' => 'value',
          '#value' => $user->id(),
        ];

        // Create list of subscription options.
        $options = [
          GROUP_SUBSCRIPTION_TYPE_DISABLED => 'Disabled',
          GROUP_SUBSCRIPTION_TYPE_IMMEDIATE => 'Immediate',
          GROUP_SUBSCRIPTION_TYPE_DAILY => 'Daily',
        ];

        $form['group_subscription'] = [
          '#type' => 'select',
          '#title' => $this->t('Subscription Type'),
          '#default_value' => $subscription,
          '#options' => $options,
        ];

        $form['parent']['actions'] = [
          '#type' => 'actions',
        ];

        $form['parent']['actions']['submit'] = [
          '#type' => 'submit',
          '#value' => $this->t('Update'),
        ];

        return $form;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $uid = $values['uid'];
    $gid = $this->group->id();
    $subscription = $values['group_subscription'];
    $groupname = $this->group->get('label')->value;

    // Prepare detailed logging.
    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('group_subscription.settings');
    $logging = $config->get('group_subscription_watchdog');

    // Check to see if this user has global group_subscription settings.
    $global_settings = \Drupal::database()->select('group_subscription_global', 'gg')
      ->fields('gg', ['uid'])
      ->condition('uid', $uid)
      ->execute()->fetchObject();

    if (empty($global_settings)) {
      // Configure global settings with defaults.
      \Drupal::database()->insert('group_subscription_global')
        ->fields([
          'uid' => $uid,
          'status' => 1,
          'sitewide' => 0,
        ])
        ->execute();

      // If detailed logging is on, log global settings changes for this user.
      if ($logging) {
        \Drupal::logger('group_subscription')->notice('Global settings for user %uid updated. Master switch set to ENABLED, site-wide announcements set to DISABLED.',
          ['%uid' => $uid]);
      }
    }

    // See if user has subscription settings saved for this group.
    $group_settings = \Drupal::database()->select('group_subscription_groups', 'g')
      ->fields('g', ['uid'])
      ->condition('uid', $uid)
      ->condition('gid', $gid)
      ->execute()->fetchObject();

    if (empty($group_settings)) {
      // Write new database record for this user / group.
      \Drupal::database()->insert('group_subscription_groups')
        ->fields([
          'uid' => $uid,
          'gid' => $gid,
          'groupname' => $groupname,
          'subscription' => $subscription,
        ])
        ->execute();
    }
    else {
      // Update the existing database record for this user / group.
      \Drupal::database()->update('group_subscription_groups')
        ->fields([
          'subscription' => $subscription,
        ])
        ->condition('uid', $uid)
        ->condition('gid', $gid)
        ->execute();
    }

    // If detailed logging is on, log group specific changes for this user.
    if ($logging) {
      $status = 'DISABLED';
      switch ($subscription) {
        case GROUP_SUBSCRIPTION_TYPE_IMMEDIATE:
          $status = 'IMMEDIATE';
          break;

        case GROUP_SUBSCRIPTION_TYPE_DAILY:
          $status = 'DAILY';
          break;
      };

      \Drupal::logger('group_subscription')->notice('Group settings for user %uid updated. Subscription for %groupname (GID: %gid) set to %subscription.',
        [
          '%uid' => $uid, 
          '%groupname' => $groupname, 
          '%gid' => $gid, 
          '%subscription' => $status,
        ]);
    }

    $this->messenger()->addStatus($this->t('Your group subscription settings for @group_name have been updated.', ['@group_name' => $groupname]));

  }

}
