<?php

namespace Drupal\group_subscription\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\group\Entity\Group;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

if (!defined('GROUP_SUBSCRIPTION_TYPE_DISABLED')) {
  define('GROUP_SUBSCRIPTION_TYPE_DISABLED', 0);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE')) {
  define('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE', 1);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_DAILY')) {
  define('GROUP_SUBSCRIPTION_TYPE_DAILY', 2);
}

/**
 * Defines a form to capture user settings for the module.
 */
class UserSettings extends ConfigFormBase {

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The core messenger service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MessengerInterface $messenger) {
    parent::__construct($config_factory);
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'group_subscription_user_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'group_subscription.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL) {
    $config = $this->config('group_subscription.settings');

    $userprofile = \Drupal::routeMatch()->getParameter('user');

    $account = User::load($userprofile);
    if (!is_object($account)) {
      return;
    }

    $result = \Drupal::database()->select('users', 'u');
    $result->leftjoin('users_field_data', 'v', 'u.uid = v.uid');
    $result->leftjoin('group_subscription_global', 'g', 'u.uid = g.uid');
    $result->fields('u', ['uid']);
    $result->fields('v', ['name', 'mail']);
    $result->fields('g', ['status', 'sitewide']);
    $result->condition('u.uid', $userprofile);
    $result->allowRowCount = TRUE;
    $global_settings = $result->execute()->fetchObject();

    $form = [];

    // Emails cannot be sent if an email address has yet to be supplied.
    if (!$global_settings->mail) {
      $url = '/user/' . $userprofile . '/edit';
      $this->messenger->addMessage($this->t('Your e-mail address must be specified on your <a href="@url">my account</a> page.', ['@url' => $url]), 'error');
    }

    // If user existed before the module was enabled, these are not set.
    if (!isset($global_settings->status)) {
      $global_settings->status = 1;
      $global_settings->sitewide = 0;
    }

    $form['status'] = [
      '#type' => 'radios',
      '#title' => $this->t('Master Switch'),
      '#default_value' => $global_settings->status,
      '#options' => [$this->t('Disabled'), $this->t('Enabled')],
      '#description' => $this->t('This option overrides all other group subscription settings below. To temporarily suspend all email notifications set this to disabled. All additional settings will be preserved.'),
    ];

    $form['sitewide'] = [
      '#type' => 'radios',
      '#title' => $this->t('Site-wide Announcements'),
      '#default_value' => $global_settings->sitewide,
      '#options' => [$this->t('Disabled'), $this->t('Enabled')],
      '#description' => $this->t('Receive email notifications about side-wide announcements.'),
    ];

    $form['group_subscription_groups'] = [
      '#type' => 'details',
      '#title' => $this->t('Group Subscriptions'),
      '#open' => TRUE,
      '#description' => $this->t('Modify subscription settings for each group membership.'),
    ];

    // Create list of subscription options.
    $options = [
      GROUP_SUBSCRIPTION_TYPE_DISABLED => 'Disabled',
      GROUP_SUBSCRIPTION_TYPE_IMMEDIATE => 'Immediate',
      GROUP_SUBSCRIPTION_TYPE_DAILY => 'Daily',
    ];

    // Retrieve stored group subscription configuration for this user.
    $result = \Drupal::database()->select('group_subscription_groups', 'g');
    $result->leftjoin('groups_field_data', 'gfd', 'g.gid = gfd.id');
    $result->fields('g', ['uid', 'gid', 'subscription']);
    $result->fields('gfd', ['label']);
    $result->condition('g.uid', $userprofile);
    $result->orderBy('groupname');
    $result->allowRowCount = TRUE;
    $group_settings = $result->execute()->fetchAll();

    // Obtain a list of group memberships for this user.
    $membership_service = \Drupal::service('group.membership_loader');
    $memberships = $membership_service->loadByUser($account);

    // Check memberships against the stored group subscription configuration.
    if (!empty($memberships)) {
      if (empty($group_settings)) {
        // No group subscriptions have been configured for this user.
        foreach ($memberships as $membership) {
          // Add a disabled subscription setting for each group membership.
          $group_settings[] = [
            'uid' => $userprofile,
            'gid' => $membership->getGroup()->id(),
            'label' => $membership->getGroup()->get('label')->value,
            'subscription' => GROUP_SUBSCRIPTION_TYPE_DISABLED,
          ];
        }
      }
      else {
        // There are already group subscriptions configured for this user.
        foreach ($memberships as $membership) {
          // Add subscription option for memberships without configuration.
          $gid = $membership->getGroup()->id();
          $configured = $this->getSubscriptionSettings($group_settings, $gid);

          if (!$configured) {
            $group_settings[] = [
              'uid' => $userprofile,
              'gid' => $gid,
              'label' => $membership->getGroup()->get('label')->value,
              'subscription' => GROUP_SUBSCRIPTION_TYPE_DISABLED,
            ];
          }
        }
      }
    }

    // Sort group subscription settings by group name alphabetically.
    $keys = array_column($group_settings, 'label');
    array_multisort($keys, SORT_ASC, $group_settings);

    // Build form element for each group membership / subscription.
    if (!empty($group_settings)) {
      // Create form field to record subscription setting for each group.
      foreach ($group_settings as $group) {
        $group = (array) $group;
        $form['group_subscription_groups']['group_' . $group['gid']] = [
          '#type' => 'select',
          '#title' => $this->t('@group_name', ['@group_name' => $group['label']]),
          '#default_value' => $group['subscription'],
          '#options' => $options,
        ];
      }
    }
    else {
      // User does not belong to any groups.
      $form['group_subscription_groups']['none'] = [
        '#markup' => $this->t('This user is not a member of any groups.'),
      ];
    }

    $form['uid'] = [
      '#type' => 'value',
      '#value' => $userprofile,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $uid = $values['uid'];

    // Prepare detailed logging.
    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('group_subscription.settings');
    $logging = $config->get('group_subscription_watchdog');

    // Write group subscription global settings for user to database.
    \Drupal::database()->delete('group_subscription_global')
      ->condition('uid', $uid)
      ->execute();

    \Drupal::database()->insert('group_subscription_global')
      ->fields([
        'uid' => $uid,
        'status' => $values['status'],
        'sitewide' => $values['sitewide'],
      ])
      ->execute();

    // If detailed logging is on, log global settings changes for this user.
    if ($logging) {
      $status = ($values['status'] == 1) ? 'ENABLED' : 'DISABLED';
      $sitewide = ($values['sitewide'] == 1) ? 'ENABLED' : 'DISABLED';

      \Drupal::logger('group_subscription')->notice('Global settings for user %uid updated. Master switch set to %status, site-wide announcements set to %sitewide.',
        ['%uid' => $uid, '%status' => $status, '%sitewide' => $sitewide]);
    }

    // Write group subscription group settings for user to database.
    foreach ($values as $key => $value) {
      if (strpos($key, 'group_') === 0) {
        $gid = str_replace('group_', '', $key);
        $groupname = Group::load($gid)->get('label')->value;

        $subscription = \Drupal::database()->select('group_subscription_groups', 'g')
          ->fields('g', ['uid'])
          ->condition('uid', $uid)
          ->condition('gid', $gid)
          ->execute()->fetchObject();

        if (empty($subscription)) {
          // Write new database record for this user / group.
          \Drupal::database()->insert('group_subscription_groups')
            ->fields([
              'uid' => $uid,
              'gid' => $gid,
              'groupname' => $groupname,
              'subscription' => $value,
            ])
            ->execute();

        }
        else {
          // Update the existing database record for this user / group.
          \Drupal::database()->update('group_subscription_groups')
            ->fields([
              'subscription' => $value,
            ])
            ->condition('uid', $uid)
            ->condition('gid', $gid)
            ->execute();

        }

        // If detailed logging is on, log group specific changes for this user.
        if ($logging) {
          $subscription = 'DISABLED';
          switch ($value) {
            case GROUP_SUBSCRIPTION_TYPE_IMMEDIATE:
              $subscription = 'IMMEDIATE';
              break;

            case GROUP_SUBSCRIPTION_TYPE_DAILY:
              $subscription = 'DAILY';
              break;
          };

          \Drupal::logger('group_subscription')->notice('Group settings for user %uid updated. Subscription for %groupname (GID: %gid) set to %subscription.',
            [
              '%uid' => $uid,
              '%groupname' => $groupname,
              '%gid' => $gid,
              '%subscription' => $subscription,
            ]);
        }
      }
    }

    $this->messenger->addMessage($this->t('Group Subscription settings saved.'));
  }

  /**
   * Checks the existing subscriptions array for the provided group id.
   *
   * @param array $settings
   *   The current group subscription settings for a given user.
   * @param string $gid
   *   The group id to search for.
   *
   * @return bool
   *   Whether the group id was found or not.
   */
  protected function getSubscriptionSettings(array $settings, string $gid) {
    foreach ($settings as $setting) {
      $setting = (array) $setting;
      if ($setting['gid'] === $gid) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
