<?php

namespace Drupal\group_subscription\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides base functionality for the Announcement Builder Queue Workers.
 */
abstract class AnnouncementBuilderBase extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The node storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $nodeStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
      EntityStorageInterface $node_storage
  ) {
    $this->nodeStorage = $node_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('entity_type.manager')->getStorage('node')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $activity = $data;

    $entity = $this->nodeStorage->load($data->entityid);
    if (is_null($entity) || !$entity->isPublished()) {
      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');

      // This entity does not exist.
      if ($logging) {
        \Drupal::logger('group_subscription')->notice('Announcement builder cannot process %entitytype. The %entitytype with ID %entityid is either unpublished or cannot be found. Processing for this announcement will be skipped.',
          ['%entitytype' => $data->entitytype, '%entityid' => $data->entityid]);
      }
      return;
    }

    // Add to announcement immediate queue.
    $this->addToImmediate($activity);

  }

  /**
   * Create an immediate email for each user subscribed to announcements.
   */
  protected function addToImmediate($activity) {
    // Get site-wide subscriptions for active users with master switch on.
    $query = \Drupal::database()->select('group_subscription_global', 'g');
    $query->join('users_field_data', 'f', 'g.uid = f.uid');
    $query->fields('g', ['uid']);
    $query->condition('g.status', 1)
      ->condition('g.sitewide', 1)
      ->condition('f.status', 1);

    $users = $query->execute()->fetchAll();
    if (count($users)) {
      // Create a new class to hold the email data.
      $email = new \stdClass();

      // Grab the announcement immediate email queue.
      $queue_factory = \Drupal::service('queue');
      $email_queue = $queue_factory->get('cron_announcement_immediate');

      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');
      $create_count = 0;

      // For each announcement subscription.
      foreach ($users as $user) {
        // Create the email object to be processed.
        $email->uid = $user->uid;
        $email->entityid = $activity->entityid;
        $email->entitytype = $activity->entitytype;
        $email->bundle = $activity->bundle;

        // Add the email object to the queue.
        $created = $email_queue->createItem($email);

        if ($created && $logging) {
          $create_count++;
        }
      }

      if ($logging) {
        \Drupal::logger('group_subscription')->notice('Created %createcount out of %attemptcount announcement email subscriptions.',
          ['%createcount' => $create_count, '%attemptcount' => count($users)]);
      }
    }
  }
}
